@@ -250,7 +250,7 @@ static struct my_option my_long_options[] =
 static const char *load_default_groups[]= { "mysqladmin","client",0 };
 
 my_bool
-get_one_option(int optid, const struct my_option *opt __attribute__((unused)),
+get_one_option(int optid, const struct my_option *opt MY_ATTRIBUTE((unused)),
 	       char *argument)
 {
   int error = 0;
@@ -537,7 +537,7 @@ int main(int argc,char *argv[])
 }
 
 
-void endprog(int signal_number __attribute__((unused)))
+void endprog(int signal_number MY_ATTRIBUTE((unused)))
 {
   interrupted=1;
 }
@@ -1054,7 +1054,7 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
         /* escape quotes if password has any special characters */
         password_len= strlen(typed_password);
         tmp= (char*) my_malloc(PSI_NOT_INSTRUMENTED, password_len*2+1, MYF(MY_WME));
-        mysql_real_escape_string(mysql, tmp, typed_password, password_len);
+        mysql_real_escape_string(mysql, tmp, typed_password, (ulong)password_len);
         typed_password= tmp;
       }
       else
@@ -1406,7 +1406,7 @@ static void print_top(MYSQL_RES *result)
 
 /* 3.rd argument, uint row, is not in use. Don't remove! */
 static void print_row(MYSQL_RES *result, MYSQL_ROW cur,
-		      uint row __attribute__((unused)))
+		      uint row MY_ATTRIBUTE((unused)))
 {
   uint i,length;
   MYSQL_FIELD *field;
@@ -1441,9 +1441,9 @@ static void print_relative_row(MYSQL_RES *result, MYSQL_ROW cur, uint row)
 }
 
 
-static void print_relative_row_vert(MYSQL_RES *result __attribute__((unused)),
+static void print_relative_row_vert(MYSQL_RES *result MY_ATTRIBUTE((unused)),
 				    MYSQL_ROW cur,
-				    uint row __attribute__((unused)))
+				    uint row MY_ATTRIBUTE((unused)))
 {
   uint length;
   ulonglong tmp;
@@ -1554,8 +1554,10 @@ static my_bool get_pidfile(MYSQL *mysql, char *pidfile)
 
   if (mysql_query(mysql, "SHOW VARIABLES LIKE 'pid_file'"))
   {
-    my_printf_error(0, "query failed; error: '%s'", error_flags,
-		    mysql_error(mysql));
+    my_printf_error(mysql_errno(mysql),
+                    "The query to get the server's pid file failed,"
+                    " error: '%s'. Continuing.", error_flags,
+                    mysql_error(mysql));
   }
   result = mysql_store_result(mysql);
   if (result)
