@@ -1,4 +1,4 @@
-/* Copyright (C) 2004, 2010 Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2006, 2015, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,35 +13,29 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
-#include "sql_priv.h"
-#include "unireg.h"
 #include "event_scheduler.h"
+
 #include "events.h"
 #include "event_data_objects.h"
 #include "event_queue.h"
 #include "event_db_repository.h"
-#include "sql_connect.h"         // init_new_connection_handler_thread
-#include "sql_acl.h"             // SUPER_ACL
+#include "auth_common.h"             // SUPER_ACL
+#include "log.h"
+#include "mysqld_thd_manager.h"      // Global_THD_manager
+#include "sql_error.h"               // Sql_condition
+#include "sql_class.h"               // THD
 
 /**
   @addtogroup Event_Scheduler
   @{
 */
 
-#ifdef __GNUC__
-#if __GNUC__ >= 2
-#define SCHED_FUNC __FUNCTION__
-#endif
-#else
-#define SCHED_FUNC "<unknown>"
-#endif
+#define LOCK_DATA()       lock_data(__func__, __LINE__)
+#define UNLOCK_DATA()     unlock_data(__func__, __LINE__)
+#define COND_STATE_WAIT(mythd, abstime, stage) \
+        cond_wait(mythd, abstime, stage, __func__, __FILE__, __LINE__)
 
-#define LOCK_DATA()       lock_data(SCHED_FUNC, __LINE__)
-#define UNLOCK_DATA()     unlock_data(SCHED_FUNC, __LINE__)
-#define COND_STATE_WAIT(mythd, abstime, msg) \
-        cond_wait(mythd, abstime, msg, SCHED_FUNC, __LINE__)
-
-extern pthread_attr_t connection_attrib;
+extern my_thread_attr_t connection_attrib;
 
 
 Event_db_repository *Event_worker_thread::db_repository;
@@ -75,9 +69,9 @@ struct scheduler_param {
 void
 Event_worker_thread::print_warnings(THD *thd, Event_job_data *et)
 {
-  MYSQL_ERROR *err;
+  const Sql_condition *err;
   DBUG_ENTER("evex_print_warnings");
-  if (thd->warning_info->is_empty())
+  if (thd->get_stmt_da()->cond_count() == 0)
     DBUG_VOID_RETURN;
 
   char msg_buf[10 * STRING_BUFFER_USUAL_SIZE];
@@ -93,18 +87,30 @@ Event_worker_thread::print_warnings(THD *thd, Event_job_data *et)
   prefix.append(et->name.str, et->name.length, system_charset_info);
   prefix.append("] ", 2);
 
-  List_iterator_fast<MYSQL_ERROR> it(thd->warning_info->warn_list());
+  Diagnostics_area::Sql_condition_iterator it=
+    thd->get_stmt_da()->sql_conditions();
   while ((err= it++))
   {
     String err_msg(msg_buf, sizeof(msg_buf), system_charset_info);
     /* set it to 0 or we start adding at the end. That's the trick ;) */
     err_msg.length(0);
     err_msg.append(prefix);
-    err_msg.append(err->get_message_text(),
-                   err->get_message_octet_length(), system_charset_info);
-    DBUG_ASSERT(err->get_level() < 3);
-    (sql_print_message_handlers[err->get_level()])("%*s", err_msg.length(),
-                                                   err_msg.c_ptr());
+    err_msg.append(err->message_text(),
+                   err->message_octet_length(), system_charset_info);
+    switch (err->severity())
+    {
+    case Sql_condition::SL_ERROR:
+      sql_print_error("%*s", static_cast<int>(err_msg.length()), err_msg.c_ptr());
+      break;
+    case Sql_condition::SL_WARNING:
+      sql_print_warning("%*s", static_cast<int>(err_msg.length()), err_msg.c_ptr());
+      break;
+    case Sql_condition::SL_NOTE:
+      sql_print_information("%*s", static_cast<int>(err_msg.length()), err_msg.c_ptr());
+      break;
+    default:
+      DBUG_ASSERT(false);
+    }
   }
   DBUG_VOID_RETURN;
 }
@@ -125,18 +131,14 @@ Event_worker_thread::print_warnings(THD *thd, Event_job_data *et)
 bool
 post_init_event_thread(THD *thd)
 {
-  (void) init_new_connection_handler_thread();
-  if (init_thr_lock() || thd->store_globals())
+  if (my_thread_init() || thd->store_globals())
   {
-    thd->cleanup();
     return TRUE;
   }
 
-  mysql_mutex_lock(&LOCK_thread_count);
-  threads.append(thd);
-  thread_count++;
-  inc_thread_running();
-  mysql_mutex_unlock(&LOCK_thread_count);
+  Global_THD_manager *thd_manager= Global_THD_manager::get_instance();
+  thd_manager->add_thd(thd);
+  thd_manager->inc_thread_running();
   return FALSE;
 }
 
@@ -152,16 +154,15 @@ post_init_event_thread(THD *thd)
 void
 deinit_event_thread(THD *thd)
 {
+  Global_THD_manager *thd_manager= Global_THD_manager::get_instance();
   thd->proc_info= "Clearing";
-  DBUG_ASSERT(thd->net.buff != 0);
-  net_end(&thd->net);
+  thd->get_protocol_classic()->end_net();
   DBUG_PRINT("exit", ("Event thread finishing"));
-  mysql_mutex_lock(&LOCK_thread_count);
-  thread_count--;
-  dec_thread_running();
+
+  thd->release_resources();
+  thd_manager->remove_thd(thd);
+  thd_manager->dec_thread_running();
   delete thd;
-  mysql_cond_broadcast(&COND_thread_count);
-  mysql_mutex_unlock(&LOCK_thread_count);
 }
 
 
@@ -183,20 +184,18 @@ void
 pre_init_event_thread(THD* thd)
 {
   DBUG_ENTER("pre_init_event_thread");
-  thd->client_capabilities= 0;
-  thd->security_ctx->master_access= 0;
-  thd->security_ctx->db_access= 0;
-  thd->security_ctx->host_or_ip= (char*)my_localhost;
-  my_net_init(&thd->net, NULL);
-  thd->security_ctx->set_user((char*)"event_scheduler");
-  thd->net.read_timeout= slave_net_timeout;
+  thd->security_context()->set_master_access(0);
+  thd->security_context()->set_db_access(0);
+  thd->security_context()->set_host_or_ip_ptr((char *) my_localhost,
+                                              strlen(my_localhost));
+  thd->get_protocol_classic()->init_net(NULL);
+  thd->security_context()->set_user_ptr(C_STRING_WITH_LEN("event_scheduler"));
+  thd->get_protocol_classic()->get_net()->read_timeout= slave_net_timeout;
   thd->slave_thread= 0;
   thd->variables.option_bits|= OPTION_AUTO_IS_NULL;
-  thd->client_capabilities|= CLIENT_MULTI_RESULTS;
-  mysql_mutex_lock(&LOCK_thread_count);
-  thd->thread_id= thd->variables.pseudo_thread_id= thread_id++;
-  mysql_mutex_unlock(&LOCK_thread_count);
+  thd->get_protocol_classic()->set_client_capabilities(CLIENT_MULTI_RESULTS);
 
+  thd->set_new_thread_id();
   /*
     Guarantees that we will see the thread in SHOW PROCESSLIST though its
     vio is NULL.
@@ -223,24 +222,31 @@ pre_init_event_thread(THD* thd)
     0  OK
 */
 
-pthread_handler_t
-event_scheduler_thread(void *arg)
+extern "C" void *event_scheduler_thread(void *arg)
 {
   /* needs to be first for thread_stack */
-  THD *thd= (THD *) ((struct scheduler_param *) arg)->thd;
+  THD *thd= ((struct scheduler_param *) arg)->thd;
   Event_scheduler *scheduler= ((struct scheduler_param *) arg)->scheduler;
   bool res;
 
   thd->thread_stack= (char *)&thd;              // remember where our stack is
 
-  mysql_thread_set_psi_id(thd->thread_id);
+  mysql_thread_set_psi_id(thd->thread_id());
 
   res= post_init_event_thread(thd);
 
   DBUG_ENTER("event_scheduler_thread");
+  my_claim(arg);
+  thd->claim_memory_ownership();
   my_free(arg);
   if (!res)
     scheduler->run(thd);
+  else
+  {
+    thd->proc_info= "Clearing";
+    thd->get_protocol_classic()->end_net();
+    delete thd;
+  }
 
   DBUG_LEAVE;                               // Against gcc warnings
   my_thread_end();
@@ -260,15 +266,18 @@ event_scheduler_thread(void *arg)
     0  OK
 */
 
-pthread_handler_t
-event_worker_thread(void *arg)
+extern "C" void *event_worker_thread(void *arg)
 {
   THD *thd;
   Event_queue_element_for_exec *event= (Event_queue_element_for_exec *)arg;
 
+  event->claim_memory_ownership();
+
   thd= event->thd;
 
-  mysql_thread_set_psi_id(thd->thread_id);
+  thd->claim_memory_ownership();
+
+  mysql_thread_set_psi_id(thd->thread_id());
 
   Event_worker_thread worker_thread;
   worker_thread.run(thd, event);
@@ -296,6 +305,18 @@ Event_worker_thread::run(THD *thd, Event_queue_element_for_exec *event)
   Event_job_data job_data;
   bool res;
 
+  DBUG_ASSERT(thd->m_digest == NULL);
+
+#ifdef HAVE_PSI_STATEMENT_INTERFACE
+  PSI_statement_locker_state state;
+  DBUG_ASSERT(thd->m_statement_psi == NULL);
+  thd->m_statement_psi= MYSQL_START_STATEMENT(& state,
+                                              event->get_psi_info()->m_key,
+                                              event->dbname.str,
+                                              event->dbname.length,
+                                              thd->charset(), NULL);
+#endif
+
   thd->thread_stack= &my_stack;                // remember where our stack is
   res= post_init_event_thread(thd);
 
@@ -324,6 +345,13 @@ Event_worker_thread::run(THD *thd, Event_queue_element_for_exec *event)
                           job_data.definer.str,
                           job_data.dbname.str, job_data.name.str);
 end:
+#ifdef HAVE_PSI_STATEMENT_INTERFACE
+  MYSQL_END_STATEMENT(thd->m_statement_psi, thd->get_stmt_da());
+  thd->m_statement_psi= NULL;
+#endif
+
+  DBUG_ASSERT(thd->m_digest == NULL);
+
   DBUG_PRINT("info", ("Done with Event %s.%s", event->dbname.str,
              event->name.str));
 
@@ -348,7 +376,7 @@ Event_scheduler::Event_scheduler(Event_queue *queue_arg)
 {
   mysql_mutex_init(key_event_scheduler_LOCK_scheduler_state,
                    &LOCK_scheduler_state, MY_MUTEX_INIT_FAST);
-  mysql_cond_init(key_event_scheduler_COND_state, &COND_state, NULL);
+  mysql_cond_init(key_event_scheduler_COND_state, &COND_state);
 }
 
 
@@ -360,28 +388,30 @@ Event_scheduler::~Event_scheduler()
 }
 
 
-/*
+
+/**
   Starts the scheduler (again). Creates a new THD and passes it to
   a forked thread. Does not wait for acknowledgement from the new
   thread that it has started. Asynchronous starting. Most of the
   needed initializations are done in the current thread to minimize
   the chance of failure in the spawned thread.
 
-  SYNOPSIS
-    Event_scheduler::start()
+  @param[out] err_no - errno indicating type of error which caused
+                       failure to start scheduler thread.
 
-  RETURN VALUE
-    FALSE  OK
-    TRUE   Error (not reported)
+  @return
+    @retval false Success.
+    @retval true  Error.
 */
 
 bool
-Event_scheduler::start()
+Event_scheduler::start(int *err_no)
 {
   THD *new_thd= NULL;
-  bool ret= FALSE;
-  pthread_t th;
+  bool ret= false;
+  my_thread_handle th;
   struct scheduler_param *scheduler_param_value;
+  ulong master_access;
   DBUG_ENTER("Event_scheduler::start");
 
   LOCK_DATA();
@@ -389,25 +419,37 @@ Event_scheduler::start()
   if (state > INITIALIZED)
     goto end;
 
+  DBUG_EXECUTE_IF("event_scheduler_thread_create_failure", {
+                  *err_no= 11;
+                  Events::opt_event_scheduler= Events::EVENTS_OFF;
+                  ret= true;
+                  goto end; });
+
   if (!(new_thd= new THD))
   {
     sql_print_error("Event Scheduler: Cannot initialize the scheduler thread");
-    ret= TRUE;
+    ret= true;
     goto end;
   }
   pre_init_event_thread(new_thd);
   new_thd->system_thread= SYSTEM_THREAD_EVENT_SCHEDULER;
-  new_thd->command= COM_DAEMON;
+  new_thd->set_command(COM_DAEMON);
 
   /*
     We should run the event scheduler thread under the super-user privileges.
     In particular, this is needed to be able to lock the mysql.event table
     for writing when the server is running in the read-only mode.
+
+    Same goes for transaction access mode. Set it to read-write for this thd.
   */
-  new_thd->security_ctx->master_access |= SUPER_ACL;
+  master_access= new_thd->security_context()->master_access();
+  new_thd->security_context()->set_master_access(master_access | SUPER_ACL);
+  new_thd->variables.tx_read_only= false;
+  new_thd->tx_read_only= false;
 
   scheduler_param_value=
-    (struct scheduler_param *)my_malloc(sizeof(struct scheduler_param), MYF(0));
+    (struct scheduler_param *)my_malloc(key_memory_Event_scheduler_scheduler_param,
+                                        sizeof(struct scheduler_param), MYF(0));
   scheduler_param_value->thd= new_thd;
   scheduler_param_value->scheduler= this;
 
@@ -415,28 +457,29 @@ Event_scheduler::start()
   DBUG_PRINT("info", ("Setting state go RUNNING"));
   state= RUNNING;
   DBUG_PRINT("info", ("Forking new thread for scheduler. THD: 0x%lx", (long) new_thd));
-  if (mysql_thread_create(key_thread_event_scheduler,
-                          &th, &connection_attrib, event_scheduler_thread,
-                          (void*)scheduler_param_value))
+  if ((*err_no= mysql_thread_create(key_thread_event_scheduler,
+                                    &th, &connection_attrib,
+                                    event_scheduler_thread,
+                                    (void*)scheduler_param_value)))
   {
     DBUG_PRINT("error", ("cannot create a new thread"));
-    state= INITIALIZED;
-    scheduler_thd= NULL;
-    ret= TRUE;
+    sql_print_error("Event scheduler: Failed to start scheduler,"
+                    " Can not create thread for event scheduler (errno=%d)",
+                    *err_no);
 
     new_thd->proc_info= "Clearing";
-    DBUG_ASSERT(new_thd->net.buff != 0);
-    net_end(&new_thd->net);
-    mysql_mutex_lock(&LOCK_thread_count);
-    thread_count--;
-    dec_thread_running();
+    new_thd->get_protocol_classic()->end_net();
+
+    state= INITIALIZED;
+    scheduler_thd= NULL;
     delete new_thd;
-    mysql_cond_broadcast(&COND_thread_count);
-    mysql_mutex_unlock(&LOCK_thread_count);
+
+    delete scheduler_param_value;
+    ret= true;
   }
+
 end:
   UNLOCK_DATA();
-
   DBUG_RETURN(ret);
 }
 
@@ -459,8 +502,8 @@ Event_scheduler::run(THD *thd)
   int res= FALSE;
   DBUG_ENTER("Event_scheduler::run");
 
-  sql_print_information("Event Scheduler: scheduler thread started with id %lu",
-                        thd->thread_id);
+  sql_print_information("Event Scheduler: scheduler thread started with id %u",
+                        thd->thread_id());
   /*
     Recalculate the values in the queue because there could have been stops
     in executions of the scheduler and some times could have passed by.
@@ -523,7 +566,7 @@ bool
 Event_scheduler::execute_top(Event_queue_element_for_exec *event_name)
 {
   THD *new_thd;
-  pthread_t th;
+  my_thread_handle th;
   int res= 0;
   DBUG_ENTER("Event_scheduler::execute_top");
   if (!(new_thd= new THD()))
@@ -547,7 +590,19 @@ Event_scheduler::execute_top(Event_queue_element_for_exec *event_name)
   if ((res= mysql_thread_create(key_thread_event_worker,
                                 &th, &connection_attrib, event_worker_thread,
                                 event_name)))
+  {
+    mysql_mutex_lock(&LOCK_global_system_variables);
+    Events::opt_event_scheduler= Events::EVENTS_OFF;
+    mysql_mutex_unlock(&LOCK_global_system_variables);
+
+    sql_print_error("Event_scheduler::execute_top: Can not create event worker"
+                    " thread (errno=%d). Stopping event scheduler", res);
+
+    new_thd->proc_info= "Clearing";
+    new_thd->get_protocol_classic()->end_net();
+
     goto error;
+  }
 
   ++started_events;
 
@@ -557,17 +612,8 @@ Event_scheduler::execute_top(Event_queue_element_for_exec *event_name)
 error:
   DBUG_PRINT("error", ("Event_scheduler::execute_top() res: %d", res));
   if (new_thd)
-  {
-    new_thd->proc_info= "Clearing";
-    DBUG_ASSERT(new_thd->net.buff != 0);
-    net_end(&new_thd->net);
-    mysql_mutex_lock(&LOCK_thread_count);
-    thread_count--;
-    dec_thread_running();
     delete new_thd;
-    mysql_cond_broadcast(&COND_thread_count);
-    mysql_mutex_unlock(&LOCK_thread_count);
-  }
+
   delete event_name;
   DBUG_RETURN(TRUE);
 }
@@ -622,7 +668,7 @@ Event_scheduler::stop()
   {
     /* Synchronously wait until the scheduler stops. */
     while (state != INITIALIZED)
-      COND_STATE_WAIT(thd, NULL, "Waiting for the scheduler to stop");
+      COND_STATE_WAIT(thd, NULL, &stage_waiting_for_scheduler_to_stop);
     goto end;
   }
 
@@ -643,21 +689,21 @@ Event_scheduler::stop()
     */
 
     state= STOPPING;
-    DBUG_PRINT("info", ("Scheduler thread has id %lu",
-                        scheduler_thd->thread_id));
+    DBUG_PRINT("info", ("Scheduler thread has id %u",
+                        scheduler_thd->thread_id()));
     /* Lock from delete */
     mysql_mutex_lock(&scheduler_thd->LOCK_thd_data);
     /* This will wake up the thread if it waits on Queue's conditional */
     sql_print_information("Event Scheduler: Killing the scheduler thread, "
-                          "thread id %lu",
-                          scheduler_thd->thread_id);
+                          "thread id %u",
+                          scheduler_thd->thread_id());
     scheduler_thd->awake(THD::KILL_CONNECTION);
     mysql_mutex_unlock(&scheduler_thd->LOCK_thd_data);
 
     /* thd could be 0x0, when shutting down */
     sql_print_information("Event Scheduler: "
                           "Waiting for the scheduler thread to reply");
-    COND_STATE_WAIT(thd, NULL, "Waiting scheduler to stop");
+    COND_STATE_WAIT(thd, NULL, &stage_waiting_for_scheduler_to_stop);
   } while (state == STOPPING);
   DBUG_PRINT("info", ("Scheduler thread has cleaned up. Set state to INIT"));
   sql_print_information("Event Scheduler: Stopped");
@@ -666,6 +712,26 @@ end:
   DBUG_RETURN(FALSE);
 }
 
+/**
+  This class implements callback for do_for_all_thd().
+  It counts the total number of living event worker threads
+  from global thread list.
+*/
+
+class Is_worker : public Do_THD_Impl
+{
+public:
+  Is_worker() : m_count(0) {}
+  virtual void operator()(THD *thd)
+  {
+    if (thd->system_thread == SYSTEM_THREAD_EVENT_WORKER)
+      m_count++;
+    return;
+  }
+  int get_count() { return m_count; }
+private:
+  int m_count;
+};
 
 /*
   Returns the number of living event worker threads.
@@ -674,19 +740,14 @@ end:
     Event_scheduler::workers_count()
 */
 
-uint
+int
 Event_scheduler::workers_count()
 {
-  THD *tmp;
-  uint count= 0;
-
+  int count= 0;
+  Is_worker is_worker;
   DBUG_ENTER("Event_scheduler::workers_count");
-  mysql_mutex_lock(&LOCK_thread_count);       // For unlink from list
-  I_List_iterator<THD> it(threads);
-  while ((tmp=it++))
-    if (tmp->system_thread == SYSTEM_THREAD_EVENT_WORKER)
-      ++count;
-  mysql_mutex_unlock(&LOCK_thread_count);
+  Global_THD_manager::get_instance()->do_for_all_thd(&is_worker);
+  count= is_worker.get_count();
   DBUG_PRINT("exit", ("%d", count));
   DBUG_RETURN(count);
 }
@@ -751,16 +812,17 @@ Event_scheduler::unlock_data(const char *func, uint line)
 */
 
 void
-Event_scheduler::cond_wait(THD *thd, struct timespec *abstime, const char* msg,
-                           const char *func, uint line)
+Event_scheduler::cond_wait(THD *thd, struct timespec *abstime, const PSI_stage_info *stage,
+                           const char *src_func, const char *src_file, uint src_line)
 {
   DBUG_ENTER("Event_scheduler::cond_wait");
   waiting_on_cond= TRUE;
-  mutex_last_unlocked_at_line= line;
+  mutex_last_unlocked_at_line= src_line;
   mutex_scheduler_data_locked= FALSE;
-  mutex_last_unlocked_in_func= func;
+  mutex_last_unlocked_in_func= src_func;
   if (thd)
-    thd->enter_cond(&COND_state, &LOCK_scheduler_state, msg);
+    thd->enter_cond(&COND_state, &LOCK_scheduler_state, stage,
+                    NULL, src_func, src_file, src_line);
 
   DBUG_PRINT("info", ("mysql_cond_%swait", abstime? "timed":""));
   if (!abstime)
@@ -770,14 +832,15 @@ Event_scheduler::cond_wait(THD *thd, struct timespec *abstime, const char* msg,
   if (thd)
   {
     /*
-      This will free the lock so we need to relock. Not the best thing to
-      do but we need to obey cond_wait()
+      Need to unlock before exit_cond, so we need to relock.
+      Not the best thing to do but we need to obey cond_wait()
     */
-    thd->exit_cond("");
+    UNLOCK_DATA();
+    thd->exit_cond(NULL, src_func, src_file, src_line);
     LOCK_DATA();
   }
-  mutex_last_locked_in_func= func;
-  mutex_last_locked_at_line= line;
+  mutex_last_locked_in_func= src_func;
+  mutex_last_locked_at_line= src_line;
   mutex_scheduler_data_locked= TRUE;
   waiting_on_cond= FALSE;
   DBUG_VOID_RETURN;
@@ -799,13 +862,13 @@ Event_scheduler::dump_internal_status()
   puts("");
   puts("Event scheduler status:");
   printf("State      : %s\n", scheduler_states_names[state].str);
-  printf("Thread id  : %lu\n", scheduler_thd? scheduler_thd->thread_id : 0);
+  printf("Thread id  : %u\n", scheduler_thd? scheduler_thd->thread_id() : 0);
   printf("LLA        : %s:%u\n", mutex_last_locked_in_func,
                                  mutex_last_locked_at_line);
   printf("LUA        : %s:%u\n", mutex_last_unlocked_in_func,
                                  mutex_last_unlocked_at_line);
   printf("WOC        : %s\n", waiting_on_cond? "YES":"NO");
-  printf("Workers    : %u\n", workers_count());
+  printf("Workers    : %d\n", workers_count());
   printf("Executed   : %lu\n", (ulong) started_events);
   printf("Data locked: %s\n", mutex_scheduler_data_locked ? "YES":"NO");
 
