@@ -1,4 +1,4 @@
-/* Copyright (c) 2008, 2015, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2008, 2016, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 #include "my_global.h"
 #include "my_thread_local.h"          // my_thread_id
 #include "mysql/psi/mysql_thread.h"   // mysql_mutex_t
+#include "handler.h"                  // enum_tx_isolation
 
 typedef struct st_mysql MYSQL;
 typedef struct st_io_cache IO_CACHE;
@@ -48,7 +49,10 @@ enum Trans_flags {
 typedef struct Trans_table_info {
   const char* table_name;
   uint number_of_primary_keys;
-  bool transactional_table;
+  /// The db_type of the storage engine used by the table
+  int db_type;
+  /// information to store if the table has foreign key with 'CASCADE' clause.
+  bool has_cascade_foreign_key;
 } Trans_table_info;
 
 /**
@@ -68,6 +72,7 @@ typedef struct Trans_table_info {
 typedef struct Trans_context_info {
   bool  binlog_enabled;
   ulong gtid_mode;               //enum values in enum_gtid_mode
+  bool log_slave_updates;
   ulong binlog_checksum_options; //enum values in enum enum_binlog_checksum_alg
   ulong binlog_format;           //enum values in enum enum_binlog_format
   // enum values in enum_transaction_write_set_hashing_algorithm
@@ -77,6 +82,8 @@ typedef struct Trans_context_info {
   // enum values in enum_mts_parallel_type
   ulong parallel_applier_type;
   ulong parallel_applier_workers;
+  bool parallel_applier_preserve_commit_order;
+  enum_tx_isolation tx_isolation;  // enum values in enum_tx_isolation
 } Trans_context_info;
 
 /**
@@ -136,7 +143,7 @@ typedef struct Trans_param {
 /**
    Transaction observer parameter initialization.
 */
-#define TRANS_PARAM_ZERO { 0, 0, 0, 0, 0, 0, {0, 0, 0}, 0, 0, 0, 0, 0, {0, 0, 0, 0, 0, 0, 0, 0, 0} }
+#define TRANS_PARAM_ZERO(trans_param_obj) memset(&trans_param_obj, 0, sizeof(Trans_param));
 
 /**
    Observes and extends transaction execution
