@@ -3,20 +3,13 @@
  All rights reserved. Use is subject to license terms.
 
  This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License, version 2.0,
- as published by the Free Software Foundation.
-
- This program is also distributed with certain software (including
- but not limited to OpenSSL) that is licensed under separate terms,
- as designated in a particular file or component or in included license
- documentation.  The authors of MySQL hereby grant you an additional
- permission to link the program and your derivative works with the
- separately licensed software that they have included with MySQL.
+ it under the terms of the GNU General Public License as published by
+ the Free Software Foundation; version 2 of the License.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License, version 2.0, for more details.
+ GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
@@ -25,7 +18,6 @@
 
 package testsuite.clusterj.model;
 
-import com.mysql.clusterj.annotation.Lob;
 import com.mysql.clusterj.annotation.PersistenceCapable;
 import com.mysql.clusterj.annotation.PrimaryKey;
 
@@ -42,7 +34,6 @@ public interface CharsetUtf8 extends CharsetModel {
     public String getMediumColumn();
     public void setMediumColumn(String value);
 
-    @Lob
     public String getLargeColumn();
     public void setLargeColumn(String value);
     
