@@ -1,4 +1,5 @@
-/* Copyright (c) 2003-2005 MySQL AB
+/*
+   Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -11,18 +12,23 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+*/
 
 #ifndef NDBFS_CONTINUEB_H
 #define NDBFS_CONTINUEB_H
 
 #include "SignalData.hpp"
 
+#define JAM_FILE_ID 130
+
+
 class NdbfsContinueB {
   /**
    * Sender(s)/Reciver(s)
    */
   friend class Ndbfs;
+  friend class VoidFs;
   friend bool printCONTINUEB_NDBFS(FILE * output, const Uint32 * theData,
 				   Uint32 len, Uint16);
 private:
@@ -32,4 +38,7 @@ private:
   };
 };
 
+
+#undef JAM_FILE_ID
+
 #endif
