@@ -1,22 +1,14 @@
 /*
-   Copyright (C) 2003-2006 MySQL AB
-    All rights reserved. Use is subject to license terms.
+   Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License, version 2.0,
-   as published by the Free Software Foundation.
-
-   This program is also distributed with certain software (including
-   but not limited to OpenSSL) that is licensed under separate terms,
-   as designated in a particular file or component or in included license
-   documentation.  The authors of MySQL hereby grant you an additional
-   permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License, version 2.0, for more details.
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -28,6 +20,9 @@
 
 #include <NodeBitmask.hpp>
 
+#define JAM_FILE_ID 94
+
+
 /**
  * This is signal is sent from "Take-Over" TC after a node crash
  * It means that the transaction was committed
@@ -42,8 +37,11 @@ class TcKeyFailConf {
    *  Reciver(s)
    */
   friend class Ndb;
+  friend class NdbImpl;
   friend class NdbTransaction;
 
+  friend class TransporterFacade;
+
 public:
   STATIC_CONST( SignalLength = 3 );
 private:
@@ -59,4 +57,7 @@ private:
   Uint32 transId2;
 };
 
+
+#undef JAM_FILE_ID
+
 #endif
