@@ -1,4 +1,5 @@
-/* Copyright (C) 2003 MySQL AB
+/*
+   Copyright (c) 2004, 2014, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -11,21 +12,17 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+*/
 
 #ifdef __sgi
 /* define on IRIX to get posix compliant vsnprintf */
 #define _XOPEN_SOURCE 500
 #endif
+
+#include <ndb_global.h>
 #include <stdio.h>
 #include <basestring_vsnprintf.h>
-#include <my_config.h>
-
-#ifdef _WINDOWS
-#define SNPRINTF_RETURN_TRUNC
-#define snprintf _snprintf
-#define vsnprintf _vsnprintf
-#endif
 
 int
 basestring_snprintf(char *str, size_t size, const char *format, ...)
@@ -38,33 +35,25 @@ basestring_snprintf(char *str, size_t size, const char *format, ...)
   return(ret);
 }
 
-#ifdef SNPRINTF_RETURN_TRUNC
-static char basestring_vsnprintf_buf[16*1024];
-#endif
 int
 basestring_vsnprintf(char *str, size_t size, const char *format, va_list ap)
 {
+  int ret;
+
   if (size == 0)
   {
-#ifdef SNPRINTF_RETURN_TRUNC
-    return vsnprintf(basestring_vsnprintf_buf,
-		     sizeof(basestring_vsnprintf_buf),
-		     format, ap);
-#else
     char buf[1];
-    return vsnprintf(buf, 1, format, ap);
-#endif
+    return basestring_vsnprintf(buf, 1, format, ap);
   }
-  {
-    int ret= vsnprintf(str, size, format, ap);
-#ifdef SNPRINTF_RETURN_TRUNC
-    if (ret == size-1 || ret == -1)
-    {
-      ret= vsnprintf(basestring_vsnprintf_buf,
-		     sizeof(basestring_vsnprintf_buf),
-		     format, ap);
-    }
-#endif
+  ret = IF_WIN(_vsnprintf,vsnprintf)(str, size, format, ap);
+  if (ret >= 0 && ret < (int)size)
     return ret;
-  }
+#ifdef _WIN32
+  if (ret < 0 && errno == EINVAL)
+    return ret;
+  // otherwise, more than size chars are needed
+  return _vscprintf(format, ap);
+#else
+  return ret;
+#endif
 }
