@@ -3,20 +3,13 @@
     All rights reserved. Use is subject to license terms.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License, version 2.0,
-   as published by the Free Software Foundation.
-
-   This program is also distributed with certain software (including
-   but not limited to OpenSSL) that is licensed under separate terms,
-   as designated in a particular file or component or in included license
-   documentation.  The authors of MySQL hereby grant you an additional
-   permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License, version 2.0, for more details.
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -691,7 +684,7 @@ flexScanThread(void* ThreadData)
       break;
     default:
       ndbout << "tType is " << tType << endl;
-      assert(false);
+      require(false);
       break;
     } // switch
     
@@ -1081,7 +1074,7 @@ static int insertRows(Ndb* pNdb, // NDB object
 	  check = MyOperations[tableCount]->insertTuple();
 	} // else if
 	else {
-	  assert(false);
+	  require(false);
 	} // else
 	
 	if (check == -1) {
@@ -1656,7 +1649,7 @@ static int verifyDeleteRows(Ndb* pNdb,
 	else {
 	  // Unexpected error
 	  ndbout << "Unexpected error during delete" << endl;
-	  assert(false);
+	  require(false);
 	} // else
 
 	pNdb->closeTransaction(MyTransaction);
