@@ -1,20 +1,13 @@
-/* Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2010, 2015, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License, version 2.0,
-  as published by the Free Software Foundation.
-
-  This program is also distributed with certain software (including
-  but not limited to OpenSSL) that is licensed under separate terms,
-  as designated in a particular file or component or in included license
-  documentation.  The authors of MySQL hereby grant you an additional
-  permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; version 2 of the License.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License, version 2.0, for more details.
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -26,7 +19,7 @@
 */
 
 #include "my_global.h"
-#include "my_pthread.h"
+#include "my_thread.h"
 #include "pfs_instr_class.h"
 #include "pfs_column_types.h"
 #include "pfs_column_values.h"
@@ -34,6 +27,8 @@
 #include "pfs_global.h"
 #include "pfs_account.h"
 #include "pfs_visitor.h"
+#include "pfs_buffer_container.h"
+#include "field.h"
 
 THR_LOCK table_esms_by_host_by_event_name::m_table_lock;
 
@@ -183,12 +178,12 @@ table_esms_by_host_by_event_name::m_share=
   table_esms_by_host_by_event_name::create,
   NULL, /* write_row */
   table_esms_by_host_by_event_name::delete_all_rows,
-  NULL, /* get_row_count */
-  1000, /* records */
+  table_esms_by_host_by_event_name::get_row_count,
   sizeof(pos_esms_by_host_by_event_name),
   &m_table_lock,
   &m_field_def,
-  false /* checked */
+  false, /* checked */
+  false  /* perpetual */
 };
 
 PFS_engine_table*
@@ -206,6 +201,12 @@ table_esms_by_host_by_event_name::delete_all_rows(void)
   return 0;
 }
 
+ha_rows
+table_esms_by_host_by_event_name::get_row_count(void)
+{
+  return global_host_container.get_row_count() * statement_class_max;
+}
+
 table_esms_by_host_by_event_name::table_esms_by_host_by_event_name()
   : PFS_engine_table(&m_share, &m_pos),
     m_row_exists(false), m_pos(), m_next_pos()
@@ -227,13 +228,14 @@ int table_esms_by_host_by_event_name::rnd_next(void)
 {
   PFS_host *host;
   PFS_statement_class *statement_class;
+  bool has_more_host= true;
 
   for (m_pos.set_at(&m_next_pos);
-       m_pos.has_more_host();
+       has_more_host;
        m_pos.next_host())
   {
-    host= &host_array[m_pos.m_index_1];
-    if (host->m_lock.is_populated())
+    host= global_host_container.get(m_pos.m_index_1, & has_more_host);
+    if (host != NULL)
     {
       statement_class= find_statement_class(m_pos.m_index_2);
       if (statement_class)
@@ -255,17 +257,16 @@ table_esms_by_host_by_event_name::rnd_pos(const void *pos)
   PFS_statement_class *statement_class;
 
   set_position(pos);
-  DBUG_ASSERT(m_pos.m_index_1 < host_max);
 
-  host= &host_array[m_pos.m_index_1];
-  if (! host->m_lock.is_populated())
-    return HA_ERR_RECORD_DELETED;
-
-  statement_class= find_statement_class(m_pos.m_index_2);
-  if (statement_class)
+  host= global_host_container.get(m_pos.m_index_1);
+  if (host != NULL)
   {
-    make_row(host, statement_class);
-    return 0;
+    statement_class= find_statement_class(m_pos.m_index_2);
+    if (statement_class)
+    {
+      make_row(host, statement_class);
+      return 0;
+    }
   }
 
   return HA_ERR_RECORD_DELETED;
@@ -274,7 +275,7 @@ table_esms_by_host_by_event_name::rnd_pos(const void *pos)
 void table_esms_by_host_by_event_name
 ::make_row(PFS_host *host, PFS_statement_class *klass)
 {
-  pfs_lock lock;
+  pfs_optimistic_state lock;
   m_row_exists= false;
 
   if (klass->is_mutable())
@@ -288,7 +289,11 @@ void table_esms_by_host_by_event_name
   m_row.m_event_name.make_row(klass);
 
   PFS_connection_statement_visitor visitor(klass);
-  PFS_connection_iterator::visit_host(host, true, true, & visitor);
+  PFS_connection_iterator::visit_host(host,
+                                      true,  /* accounts */
+                                      true,  /* threads */
+                                      false, /* THDs */
+                                      & visitor);
 
   if (! host->m_lock.end_optimistic_lock(&lock))
     return;
